-- Author       : Jan Kenneth Bekkeng
-- Company      : Univeristy of Oslo
-- Project      : FYS 4220/9220
-- Date         : 12.08.2010
-- Last rev     : 
-- Version      : 1
-- Function     : Debouncer


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity Debouncer is 
  port
  (
    clk         : in    std_logic;
    reset       : in    std_logic;
    SwitchIn    : in    std_logic;
    SwitchOut   : out   std_logic
  );
end Debouncer;



architecture Debouncer_ARCH of Debouncer is


-- The system clock (clk) is assumed to be 1 MHz
constant SampleCountMax   : integer := 499;  -- puls med en frekvens p ca 2 kHz
                                            -- (T = 1 / f br ligge rundt 500 us)
constant DebounceCountMax : integer := 20;  -- pulser lenger enn 10 ms detecteres
                                            -- (br ligge p ca 10 ms)

signal q1               : std_logic;
signal q2               : std_logic;   -- Output from synchronizer (no metastates);
signal sample_cnt       : std_logic_vector(8 downto 0);   -- 2^9 = 512 (needs to count >= 499)    
signal debounce_cnt     : std_logic_vector(4 downto 0);   -- 2^5 = 32 (needs to count >= 20)   
signal sample           : std_logic;

  
begin

  

SYNCHRONIZER:
process(clk)
begin
  if rising_edge(clk) then
    q1 <= SwitchIn;
    q2 <= q1;
  end if;
end process;

    
SAMPLEPULS:
process(clk)
begin
  if rising_edge(clk) then
    if (reset = '1') then                        -- synch. reset
      sample_cnt <= (others => '0');
    else
      if (sample_cnt = SampleCountMax) then
        sample_cnt <= (others => '0');
        sample <= '1';
      else
        sample_cnt <= sample_cnt + 1;
        sample <= '0';
      end if;
    end if;
  end if;
end process;
 

DEBOUNCER:
process(clk)                     
begin
  if rising_edge(clk) then
    if (reset = '1') then                       -- synch. reset
      debounce_cnt <= (others => '0');
    elsif (q2 = '0') then                       -- Switch not activated
      debounce_cnt <= (others => '0');          -- Reset counter
      SwitchOut <= '0';
    else
      if (sample = '1') then                    -- (Switch is activated)
        if (debounce_cnt = DebounceCountMax) then
          SwitchOut <= '1';
        else
          SwitchOut <= '0';
          debounce_cnt <= debounce_cnt + 1;
        end if;
      end if;
    end if;
  end if;
end process;
   
end Debouncer_ARCH;
