-- Author       : Jan Kenneth Bekkeng
-- Company      : University of Oslo
-- File name    : counters_lab1.vhd 
-- Date         : 12.08.2010
-- Version      : 01
-- Project      : FYS 4220/9220
-- Function     : Word counter and bit counter


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity counters_lab1 is
  
  port
    (
    clk		: in		std_logic; 	
	reset	: in		std_logic;
	bit_cnt	: buffer	std_logic_vector(2 downto 0); -- 8 bits/word
	word_cnt: buffer	std_logic_vector(6 downto 0)  -- 96 words/frame (2^7 = 128)
    );
    
end counters_lab1;


architecture part3 of counters_lab1 is

constant 	WordsPerFrame 	: integer := 96; 
signal 		WordStrobe		: std_logic;


begin

BITCOUNTER:
process(clk, reset)
begin 
  if (reset = '1') then            -- async reset
    bit_cnt <= (others  => '0');
  elsif falling_edge(clk) then            
    bit_cnt <= bit_cnt + 1;
  end if;
end process;


WORDSYNC:
process(clk)
begin
  if rising_edge(clk) then             
    if (bit_cnt = 7) then
      WordStrobe <= '1';
    else
      WordStrobe <= '0';
    end if;
  end if;
end process;


WORDCOUNTER:
process(clk, reset)
begin
  if (reset = '1') then		-- async reset
    word_cnt <= (others  => '0');
  elsif falling_edge(clk) then            
    if (WordStrobe = '1') then
      if (word_cnt < WordsPerFrame-1) then
        word_cnt <= word_cnt + 1;
      else
        word_cnt <= (others => '0');
      end if;
    end if;
  end if;
end process;

end part3;
